#import "Controller.h"

@implementation Controller


- (void)awakeFromNib
{
	NSRect		rect;
	NSView		*tempView;
	
	rect = [box bounds];
	
	// Set up intermediary views and stick the views inside it. These views
	// acts as viewports to the "real" views.
	tempView = [[NSView alloc] initWithFrame:rect];
	[tempView addSubview:view1];
	[tempView setAutoresizingMask:[box autoresizingMask]];
	[box addSubview:tempView];

	rect = [tempView bounds];
	[view1 setFrame:rect];
	view1 = tempView;

	tempView = [[NSView alloc] initWithFrame:rect];
	[tempView addSubview:view2];
	[tempView setAutoresizingMask:[box autoresizingMask]];	
	[box addSubview:tempView];
	
	rect = [tempView bounds];	
	[tempView setHidden:YES];
	[view2 setFrame:rect];
	view2 = tempView;
}

- (void)prepSubviewOfView:(NSView *)view
{
	NSView		*subview;
	
	subview = [[view subviews] objectAtIndex:0];
	[subview setFrameOrigin:NSZeroPoint];
	// Reset the mask and let each animation turn on whatever resizing options it needs
	[subview setAutoresizingMask:NSViewNotSizable];
}

- (void)resetSubviewOfView:(NSView *)view
{
	NSView		*subview;
	
	subview = [[view subviews] objectAtIndex:0];
	[subview setFrameOrigin:NSZeroPoint];
	// Allow the views to resize properly now that the animation is done.
	[subview setAutoresizingMask:[box autoresizingMask]];
}

- (IBAction)doTransition:sender
{
	AnimationType		transitionType;
	NSViewAnimation		*animation;
	NSRect				rect;
	NSView				*oldView, *newView;

	transitionType = [transitionSelector selectedTag];
	rect = [box bounds];
	
	if ([view1 isHidden])
	{
		newView = view1;
		oldView = view2;
	}
	else
	{
		newView = view2;
		oldView = view1;
	}
	
	// Unset the first responder. Can cause drawing artifacts since the blue glow extends beyond the view's bounds.
	[[box window] makeFirstResponder:nil];
	
	// Turn off all autoresizing for now. Will be twiddling these for different effects.
	[self prepSubviewOfView:newView];
	[self prepSubviewOfView:oldView];
	
	switch (transitionType)
	{
		case Dissolve:
			[newView setFrame:rect];
			// Note: view does not need to be unhidden manually as in the other cases. This is because the fade in
			// effect will do it for you.

			animation = [[NSViewAnimation alloc] initWithViewAnimations:
							[NSArray arrayWithObjects:
								// Old view is set to fade out
								[NSDictionary dictionaryWithObjectsAndKeys:
									oldView, NSViewAnimationTargetKey, NSViewAnimationFadeOutEffect, NSViewAnimationEffectKey, nil],
								// New view is set to fade in
								[NSDictionary dictionaryWithObjectsAndKeys:
									newView, NSViewAnimationTargetKey, NSViewAnimationFadeInEffect, NSViewAnimationEffectKey, nil],
				nil]];
			break;
		case MoveIn:
			// Set the old view to be resizable on the right side. It will appear as if it's sitting still
			[[[oldView subviews] objectAtIndex:0] setAutoresizingMask:NSViewMaxXMargin];
			
			// Set the new view to be out of bounds on the right side, ready to be animated in
			[newView setFrame:NSMakeRect(NSMaxX(rect), NSMinY(rect), NSWidth(rect), NSHeight(rect))];
			// If a previous animation resulted in a zero frame, it set it to hidden. We have to unhide it manually.
			[newView setHidden:NO];
			
			animation = [[NSViewAnimation alloc] initWithViewAnimations:
				[NSArray arrayWithObjects:
					// The left "viewport" shrinks to zero-width on the left side
					[NSDictionary dictionaryWithObjectsAndKeys:
						oldView, NSViewAnimationTargetKey,
						[NSValue valueWithRect:NSMakeRect(NSMinX(rect), NSMinY(rect), 0.0, NSHeight(rect))], NSViewAnimationEndFrameKey, nil],
					// New view frame is just the viewable area. Since it is currently out of bounds, it will appear to slide
					// in bounds.
					[NSDictionary dictionaryWithObjectsAndKeys:
						newView, NSViewAnimationTargetKey,
						[NSValue valueWithRect:rect], NSViewAnimationEndFrameKey, nil],
					nil]];
			break;
		case Push:
			// Set old view to be resizable on the left side. It will appear to move to the left since the right margin is
			// fixed.
			[[[oldView subviews] objectAtIndex:0] setAutoresizingMask:NSViewMinXMargin];			
			
			// Set the new view to be out of bounds on the right side, ready to be animated in			
			[newView setFrame:NSMakeRect(NSMaxX(rect), NSMinY(rect), NSWidth(rect), NSHeight(rect))];
			// If a previous animation resulted in a zero frame, it set it to hidden. We have to unhide it manually.
			[newView setHidden:NO];
			
			animation = [[NSViewAnimation alloc] initWithViewAnimations:
				[NSArray arrayWithObjects:
					// Old view frame gets moved to the left, out of bounds.
					[NSDictionary dictionaryWithObjectsAndKeys:
						oldView, NSViewAnimationTargetKey,
						[NSValue valueWithRect:NSMakeRect(NSMinX(rect) - NSWidth(rect), NSMinY(rect), NSWidth(rect), NSHeight(rect))], NSViewAnimationEndFrameKey, nil],
					// New view frame is just the viewable area. Since it is currently out of bounds, it will appear to slide
					// in bounds. Note, this is the same as in the "Move In" case.
					[NSDictionary dictionaryWithObjectsAndKeys:
						newView, NSViewAnimationTargetKey,
						[NSValue valueWithRect:rect], NSViewAnimationEndFrameKey, nil],
					nil]];
			break;
		case Reveal:
			// Set old view to be resizable on the left side. It will appear to move to the left since the right margin is
			// fixed.
			[[[oldView subviews] objectAtIndex:0] setAutoresizingMask:NSViewMinXMargin];
			
			// Set new view to be resizable on the right side. It will appear to sit still.
			[[[newView subviews] objectAtIndex:0] setAutoresizingMask:NSViewMinXMargin];
			
			// Make the right viewport zero-width view on the right side. It will reveal the view as it expands to the left.
			[newView setFrame:NSMakeRect(NSMaxX(rect), NSMinY(rect), 0.0, NSHeight(rect))];
			[newView setHidden:NO];
			
			// Make the subview such that it's right edge is flush with its superview/viewport.
			[[[newView subviews] objectAtIndex:0] setFrame:NSMakeRect(NSMinX(rect) - NSWidth(rect), NSMinY(rect), NSWidth(rect), NSHeight(rect))];
			
			animation = [[NSViewAnimation alloc] initWithViewAnimations:
				[NSArray arrayWithObjects:
					// The left "viewport" shrinks to zero-width
					[NSDictionary dictionaryWithObjectsAndKeys:
						oldView, NSViewAnimationTargetKey,
						[NSValue valueWithRect:NSMakeRect(NSMinX(rect), NSMinY(rect), 0.0, NSHeight(rect))], NSViewAnimationEndFrameKey, nil],
					// The right view port expands to encompass the full bounds.
					[NSDictionary dictionaryWithObjectsAndKeys:
						newView, NSViewAnimationTargetKey,
						[NSValue valueWithRect:rect], NSViewAnimationEndFrameKey, nil],
					nil]];
			break;
		case Wipe:
			// Set the old view to be resizable on the right side. It will appear as if it's sitting still
			[[[oldView subviews] objectAtIndex:0] setAutoresizingMask:NSViewMaxXMargin];
			
			// Set new view to be resizable on the right side. It will appear to sit still.
			[[[newView subviews] objectAtIndex:0] setAutoresizingMask:NSViewMinXMargin];
			
			// Make the right viewport zero-wdith
			[newView setFrame:NSMakeRect(NSMaxX(rect), NSMinY(rect), 0.0, NSHeight(rect))];
			[newView setHidden:NO];
			
			// Make the subview such that it's right edge is flush with its superview/viewport.
			[[[newView subviews] objectAtIndex:0] setFrame:NSMakeRect(NSMinX(rect) - NSWidth(rect), NSMinY(rect), NSWidth(rect), NSHeight(rect))];

			animation = [[NSViewAnimation alloc] initWithViewAnimations:
				[NSArray arrayWithObjects:
					// The left "viewport" shrinks to zero-width
					[NSDictionary dictionaryWithObjectsAndKeys:
						oldView, NSViewAnimationTargetKey,
						[NSValue valueWithRect:NSMakeRect(NSMinX(rect), NSMinY(rect), 0.0, NSHeight(rect))], NSViewAnimationEndFrameKey, nil],
					// The right view port expands to encompass the full bounds.
					[NSDictionary dictionaryWithObjectsAndKeys:
						newView, NSViewAnimationTargetKey,
						[NSValue valueWithRect:rect], NSViewAnimationEndFrameKey, nil],
					nil]];
			break;
		default:
			NSLog(@"Unknown transition type");
	}

	[animation setAnimationBlockingMode:NSAnimationBlocking];
	[animation startAnimation];
	[animation release];
	
	// Not all the animations above result in the old view being hidden so do it here
	[oldView setHidden:YES];
	
	[self resetSubviewOfView:oldView];
	[self resetSubviewOfView:newView];
}

@end
